#!/usr/bin/env python
# -*- coding: utf-8  -*-

import logging as lg
import logging.config as lgcfg
import os
import shutil

default_config_filename = "logging-default.cfg"

class Logger(object):
	'''生成一个Logger

	lgr = Logger()

	Args:
		config_filepath: 配置文件路径。如果参数为空，且当前工作目录没有默认配置文件（logging-default.cfg），则将包中配置文件复制到工作目录。
		logger_name: 使用Logger的名称。默认使用root。
	
	Returns:
		返回一个Logger对象。

	Raises:
		

	'''
	def __init__(self, config_filepath = None, logger_name = None):
		if config_filepath is None:
			if not os.path.exists(default_config_filename): 
				src_script_dir = os.path.split(os.path.realpath(__file__))[0]
				exec_script_dir = os.getcwd()
				src_config = os.path.join(src_script_dir, default_config_filename)
				exec_config = os.path.join(exec_script_dir, default_config_filename)
				shutil.copy(src_config, exec_config)
			config_filepath = default_config_filename
		lgcfg.fileConfig(config_filepath)
		self.logger = lg.getLogger(logger_name)

	
	def debug(self, obj):
		self.logger.debug(obj)

	def info(self, obj):
		self.logger.info(obj)
	
	def warning(self, obj):
		self.logger.warning(obj)
	
	def error(self, obj):
		self.logger.error(obj)
	
	def critical(self, obj):
		self.logger.critical(obj)

	


#!/usr/bin/env python
# -*- coding: utf-8  -*-

"""
execute the unittest file in project root directory.
"""
import sys
sys.path.insert(0, r"../datalet")

import unittest

import pandas as pd

from datalet.data import *
from datalet.storage import *
from datalet.storage.dbtable_storage import *
import datalet.storage.facade as sfd
from sqlalchemy import create_engine

mysql_db_url = create_mysql_dburl("localhost", "abc", "abc", "123456", port = 8866)
engine = create_engine(mysql_db_url)

#sql_yueyuefa = "" \
# " select * from (" \
# " select ap.id, ap.uid, ap.corp, cb.status, ap.fund, ap.refund, cb.paidprincipal, cb.principal, cb.paidinterest, cb.interest, ap.applytime, ap.canceltime, ap.period, ap.applyuid, cb.beginday, cb.endday from adv_payment ap left join credit_bill cb on ap.id = cb.advid" \
# " union" \
# " select ap.id, ap.uid, ap.corp, cb.status, ap.fund, ap.refund, cb.paidprincipal, cb.principal, cb.paidinterest, cb.interest, ap.applytime, ap.canceltime, ap.period, ap.applyuid, cb.beginday, cb.endday from adv_payment ap left join credit_bill cb on ap.id = cb.advid" \
# " ) full_table "
sql_yueyuefa = "" \
" select ai.acct_name, full_table.* from (" \
" 	(select ap.id, ap.uid, ap.corp, cb.status, ap.fund, " \
" 	ap.refund, cb.paidprincipal, cb.principal, cb.paidinterest, cb.interest, " \
" 	ap.applytime, ap.canceltime, ap.period, ap.applyuid, cb.beginday, cb.endday " \
" 	from yueyuefa.adv_payment ap left join yueyuefa.credit_bill cb on ap.id = cb.advid)" \
" 	union" \
" 	(select ap.id, ap.uid, ap.corp, cb.status, ap.fund, " \
" 	ap.refund, cb.paidprincipal, cb.principal, cb.paidinterest, cb.interest, " \
" 	ap.applytime, ap.canceltime, ap.period, ap.applyuid, cb.beginday, cb.endday " \
" 	from yueyuefa.adv_payment ap right join yueyuefa.credit_bill cb on ap.id = cb.advid)" \
" ) full_table " \
" left join yueyuefa.ods_fc_acct_info ai on ai.acct_id = full_table.uid" \

result = engine.execute(text(sql_yueyuefa), {})
ret_all = result.fetchall()
#first = ret_all[:5]
#print(first)

dt_yueyuefa = DataTable("table_yueyuefa",
	DataColumn(name = "id", caption = "信用加款流水号"),
	DataColumn(name = "acct_name", caption = "账户名称"),
	DataColumn(name = "uid", caption = "账户ID"),
	DataColumn(name = "corp", caption = "所属分公司"),
	DataColumn(name = "status", caption = "当前支付状态"),
	DataColumn(name = "fund", caption = "信用加款金额"),
	DataColumn(name = "refund", caption = "信用退款金额"),
	DataColumn(name = "paidprincipal", caption = "已还信用款额"),
	DataColumn(name = "principal", caption = "未还信用款额"),
	DataColumn(name = "paidinterest", caption = "已还手续费"),
	DataColumn(name = "interest", caption = "未还手续费"),
	DataColumn(name = "applytime", caption = "信用加款时间"),
	DataColumn(name = "canceltime", caption = "还款截止时间"),
	DataColumn(name = "period", caption = "授信账期"),
	DataColumn(name = "applyuid", caption = "申请客服"),
	DataColumn(name = "beginday", caption = "账单开始时间"),
	DataColumn(name = "endday", caption = "账单结束时间"))

for row in ret_all:
	dt_yueyuefa.append(dict(row))

print(len(dt_yueyuefa))

sfd.write_file(r"D:\yueyuefa.xlsx", data = dt_yueyuefa, sheetIndex = 0, force = True, overwrite=True)

# es_yueyuefa = sfd.create_file(r"D:\yueyuefa.xlsx", sheetIndex = 0, force = True)
# if es_yueyuefa.exists_file():
	# es_yueyuefa.remove()
# es_yueyuefa.write(data = dt_yueyuefa)



#!/usr/bin/env python
# -*- coding: utf-8  -*-

import datalet.storage._utils as _utils


def exists_file(location, storage_type = None, **kwargs):
	if kwargs is None or len(kwargs) == 0:
		kwargs = {}
	kwargs["location"] = location
	construct_func = _utils.get_storage_func(location, storage_type)
	storage_obj = _utils.call_func(construct_func, kwargs)
	return _utils.call_func(storage_obj.exists, kwargs)


def create_file(location,  storage_type = None, force = False, **kwargs):
	if kwargs is None or len(kwargs) == 0:
		kwargs = {}
	kwargs["location"] = location
	kwargs["force"] = force
	construct_func = _utils.get_storage_func(location, storage_type)
	storage_obj = _utils.call_func(construct_func, kwargs)
	return _utils.call_func(storage_obj.create, kwargs)


def clear_file(location,  storage_type = None, force = False,**kwargs):
	if kwargs is None or len(kwargs) == 0:
		kwargs = {}
	kwargs["location"] = location
	kwargs["force"] = force
	construct_func = _utils.get_storage_func(location, storage_type)
	storage_obj = _utils.call_func(construct_func, kwargs)
	return _utils.call_func(storage_obj.clear, kwargs)


def remove_file(location,  storage_type = None, force = False, **kwargs):
	if kwargs is None or len(kwargs) == 0:
		kwargs = {}
	kwargs["location"] = location
	kwargs["force"] = force
	construct_func = _utils.get_storage_func(location, storage_type)
	storage_obj = _utils.call_func(construct_func, kwargs)
	return _utils.call_func(storage_obj.remove, kwargs)


def read_file(location, line_limit = -1, storage_type = None, **kwargs):
	if kwargs is None or len(kwargs) == 0:
		kwargs = {}
	kwargs["location"] = location
	kwargs["line_limit"] = line_limit
	construct_func = _utils.get_storage_func(location, storage_type)
	storage_obj = _utils.call_func(construct_func, kwargs)
	return _utils.call_func(storage_obj.read, kwargs)


def write_file(location, storage_type = None, data = None, overwrite = True, **kwargs):
	if kwargs is None or len(kwargs) == 0:
		kwargs = {}
	kwargs["location"] = location
	kwargs["data"] = data
	kwargs["overwrite"] = overwrite
	construct_func = _utils.get_storage_func(location, storage_type)
	storage_obj = _utils.call_func(construct_func, kwargs)
	return _utils.call_func(storage_obj.write, kwargs)


def create_db(location,  storage_type = None, force = False, **kwargs):
	raise NotImplementedError


def create_temp_storage(location, force = False, **kwargs):
	raise NotImplementedError



#!/usr/bin/env python
# -*- coding: utf-8  -*-

"""
execute the unittest file in project root directory.
"""
import sys
sys.path.insert(0, r"../datalet")

import os
import unittest

from sqlalchemy import Table, Column, Integer, String, MetaData, TIMESTAMP, DECIMAL, BigInteger

from datalet.data import *
from datalet.storage.dbtable_storage import *
from datalet.storage.excel_storage import ExcelStorage
from tests.testing import Testing
import datalet.utils.sql_utils as sql_utils

metadata = MetaData()

class DbTableStorageTest(unittest.TestCase):

	def setUp(self):
		self.tmpdir = r"tests/test_data/tmp/"
		self.classname = self.__class__.__name__
		self.separator = "$"
		self.ext = ".csv"

		self.host = "localhost"
		self.dbname = "testdb"
		self.user = "pgadmin"
		self.pwd = "pgadmin."

		#self.host = "localhost"
		#self.dbname = "testdb"
		#self.user = "pgdba"
		#self.pwd = "pgdba"


		self.table = Table("test_table", metadata,
					 Column("code", String(100), nullable = False),
					 Column("name", String(100), nullable = True),
					 Column("create_date", TIMESTAMP, nullable = False),
					 extend_existing=True,
					 schema = "test_schema")

	def tearDown(self):
		pass

	# def test_create_pg_dburl(self):
	# 	url = sql_utils.create_pg_sqlalchemy_dburl(host="localhost", dbname="testdb", user="pgadmin", pwd="pgadmin123")
	# 	self.assertEqual("postgresql+psycopg2://pgadmin:pgadmin123@localhost:5432/testdb", url)
    #
	# def test_create_table_force_true(self):
	# 	dburl = sql_utils.create_pg_sqlalchemy_dburl(host = self.host, dbname = self.dbname, user = self.user, pwd = self.pwd)
	# 	s = DbTableStorage(dburl, table=self.table)
	# 	self.assertEqual(s.create(force = True), None)
    #
	# def test_create_table_force_false(self):
	# 	dburl = sql_utils.create_pg_sqlalchemy_dburl(host = self.host, dbname = self.dbname, user = self.user, pwd = self.pwd)
	# 	s = DbTableStorage(dburl, table=self.table)
	# 	with self.assertRaises(StorageExistsError):
	# 		s.create(force = False)
    #
	# def test_write_data_overwrite_true(self):
	# 	dburl = sql_utils.create_pg_sqlalchemy_dburl(host = self.host, dbname = self.dbname, user = self.user, pwd = self.pwd)
	# 	s = DbTableStorage(dburl, table=self.table)
	# 	data = [{"code": "ACF1001", "name": "lisi", "create_date": "2016-01-01"},
	# 	  {"code": "ACF1002", "name": "zhangsan", "create_date": "2016-01-02"},
	# 	  {"code": "ACF1003", "name": "wangwu", "create_date": "2016-01-03"}]
	# 	s.write(data, overwrite = True)
	# 	#s.write(data, overwrite = False)
	# 	#s.write(data, overwrite = True)
    #
	# def test_write_data_by_batch(self):
	# 	dburl = sql_utils.create_pg_sqlalchemy_dburl(host = self.host, dbname = self.dbname, user = self.user, pwd = self.pwd)
	# 	s = DbTableStorage(dburl, table=self.table)
	# 	s.create(force = True)
	# 	data = []
	# 	for i in range(0, 100):
	# 		d = {"code": "CODE_" + str(i), "name": "NAME_" + str(i), "create_date": "2016-01-01"}
	# 		data.append(d)
	# 	batch_rows_count = 32
	# 	s.write(data, batch_rows_count = batch_rows_count)
    #
	# def test_write_data_transaction(self):
	# 	dburl = sql_utils.create_pgsql_sqlalchemy_dburl(host = self.host, dbname = self.dbname, user = self.user, pwd = self.pwd)
	# 	s = DbTableStorage(dburl, table=self.table)
	# 	if not s.exists():
	# 		s.create(force = False)
	# 	data = []
	# 	for i in range(0, 100):
	# 		d = {"code": "CODE_" + str(i), "name": "NAME_" + str(i), "create_date": "2016-01-01"}
	# 		data.append(d)
	# 	batch_rows_count = 20
	# 	s.write(data, batch_rows_count = batch_rows_count)
    #
	# def test_read_no_limit(self):
	# 	dburl = sql_utils.create_pg_sqlalchemy_dburl(host = self.host, dbname = self.dbname, user = self.user, pwd = self.pwd)
	# 	s = DbTableStorage(dburl, table=self.table)
	# 	for row in s.read():
	# 		print(row)

	# def test_write_large_data(self):
	# 	import pandas as pd
	# 	excel = ExcelStorage(location = r"D:\svnserver\业务数据\数据库同步数据\交易数据交易数据_1607_new.xlsx", sheet_name = "交易")
	# 	#excel = ExcelStorage(filepath = r"D:\正单_TEST.xlsx", sheetIndex = 1)
	# 	datatable = excel.read()
	# 	datatable.shift_rows_to_header(1)
	# 	df_data = datatable.to_pandas_dataframe()
	# 	print("Got dataframe.")
	# 	df_data = df_data.rename(columns = {"会计时间": "fiscal_date",
	# 								  "商户ID": "mer_id",
	# 								  "商户名称": "mer_name",
	# 								  "交易名称": "trade_name",
	# 								  "交易类型": "trade_type",
	# 								  "交易状态": "trade_status",
	# 								  "模式名称": "mode_name",
	# 								  "支付工具": "pay_tools",
	# 								  "业务名称": "pay_biz_name",
	# 								  "卡类型": "cardtype",
	# 								  "卡类型名称": "cardtype_name",
	# 								  "发卡行": "send_bank_code",
	# 								  "收单行": "recv_bank_code",
	# 								  "银行商户号": "bank_mer_code",
	# 								  "银行清算时间": "bank_clr_time",
	# 								  "笔数": "orders_count",
	# 								  "金额": "orders_money",
	# 								  "商户手续费": "mer_fee",
	# 								  "业务名称_处理后": "biz_name_proced",
	# 								  "银行名称_处理后": "bank_name_proced",
	# 								  "卡类型_处理后": "cardtype_name_proced",
	# 								  "名义费率_处理后": "nominal_rate",
	# 								  "名义成本_处理后": "nominal_cost",
	# 								  "1级所属类型_处理后": "biz_level_1st",
	# 								  "2级所属类型_处理后": "biz_level_2nd",
	# 								  "3级所属类型_处理后": "biz_level_3rd"})
	# 	df_wanted = df_data.loc[:, ["fiscal_date",
	# 					  "mer_id",
	# 					  "mer_name",
	# 					  "trade_name",
	# 					  "trade_type",
	# 					  "trade_status",
	# 					  "mode_name",
	# 					  "pay_tools",
	# 					  "pay_biz_name",
	# 					  "cardtype",
	# 					  "cardtype_name",
	# 					  "send_bank_code",
	# 					  "recv_bank_code",
	# 					  "bank_mer_code",
	# 					  "bank_clr_time",
	# 					  "orders_count",
	# 					  "orders_money",
	# 					  "mer_fee",
	# 					  "biz_name_proced",
	# 					  "bank_name_proced",
	# 					  "cardtype_name_proced",
	# 					  "nominal_rate",
	# 					  "nominal_cost",
	# 					  "biz_level_1st",
	# 					  "biz_level_2nd",
	# 					  "biz_level_3rd"]]
	# 	df_wanted["orders_count"] = df_wanted["orders_count"].astype(object)
	# 	print(df_wanted)
    #
	# 	tb = Table("large_table", metadata,
	# 				 Column("fiscal_date", TIMESTAMP, nullable = True),
	# 				 Column("mer_id", String(100), nullable = True),
	# 				 Column("mer_name", String(100), nullable = True),
	# 				 Column("trade_name", String(100), nullable = True),
	# 				 Column("trade_type", String(100), nullable = True),
	# 				 Column("trade_status", String(100), nullable = True),
	# 				 Column("mode_name", String(100), nullable = True),
	# 				 Column("pay_tools", String(100), nullable = True),
	# 				 Column("pay_biz_name", String(100), nullable = True),
	# 				 Column("cardtype", String(100), nullable = True),
	# 				 Column("cardtype_name", String(100), nullable = True),
	# 				 Column("send_bank_code", String(100), nullable = True),
	# 				 Column("recv_bank_code", String(100), nullable = True),
	# 				 Column("bank_mer_code", String(100), nullable = True),
	# 				 Column("bank_clr_time", TIMESTAMP, nullable = True),
	# 				 Column("orders_count", BigInteger, nullable = True),
	# 				 Column("orders_money", DECIMAL, nullable = True),
	# 				 Column("mer_fee", DECIMAL, nullable = True),
	# 				 Column("biz_name_proced", String(100), nullable = True),
	# 				 Column("bank_name_proced", String(100), nullable = True),
	# 				 Column("cardtype_name_proced", String(100), nullable = True),
	# 				 Column("nominal_rate", String(100), nullable = True),
	# 				 Column("nominal_cost", DECIMAL, nullable = True),
	# 				 Column("biz_level_1st", String(100), nullable = True),
	# 				 Column("biz_level_2nd", String(100), nullable = True),
	# 				 Column("biz_level_3rd", String(100), nullable = True),
	# 				 extend_existing=True,
	# 				 schema = "test_schema")
	# 	dburl = sql_utils.create_pg_sqlalchemy_dburl(host = self.host, dbname = self.dbname, user = self.user, pwd = self.pwd)
	# 	s = DbTableStorage(dburl, table = tb)
	# 	if not s.exists():
	# 		s.create(force = False)
	# 	#print("start to build dict.")
	# 	#df_wanted["orders_count"] = df_wanted["orders_count"].astype(object)
	# 	#df_wanted = df_wanted.applymap(proc_nan)
	# 	#print(df_wanted[:5].to_dict(orient = "records"))
	# 	#print(df_wanted.dtypes)
	# 	#dict_data = df_wanted.to_dict(orient = "records")
	# 	#print("succeed in building dict.")
	# 	new_table = DataTable()
	# 	new_table.from_pandas_dataframe(df_wanted)
	# 	dict_data = new_table.to_dict(null_expr = None)
	# 	s.write(dict_data, batch_rows_count = 100)
	# 	print("succeed in writing to db.")


def suite():
	suite = unittest.TestSuite()
	#suite.addTest(DbTableStorageTest("test_create_pg_dburl"))
	#suite.addTest(DbTableStorageTest("test_create_table_force_true"))
	#suite.addTest(DbTableStorageTest("test_create_table_force_false"))
	#suite.addTest(DbTableStorageTest("test_write_data_overwrite_true"))
	#suite.addTest(DbTableStorageTest("test_write_data_by_batch"))
	#suite.addTest(DbTableStorageTest("test_write_data_transaction"))
	#suite.addTest(DbTableStorageTest("test_read_no_limit"))
	suite.addTest(DbTableStorageTest("test_write_large_data"))
	return suite

if __name__ == "__main__":
	unittest.main(defaultTest = "suite")



#!/usr/bin/env python
# -*- coding: utf-8  -*-

class StorageExistedError(Exception):

	def __init__(self, location):
		self.location = location

class StorageNotFoundError(Exception):

	def __init__(self, location):
		self.location = location

class UnmatchExtensionError(Exception):

	def __init__(self, extname):
		self.extname = extname

class ForeignRelationExistedError(Exception):

	def __init__(self, location):
		self.location = location

class ArgumentsAbsenceError(Exception):

	def __init__(self, argname):
		self.argname = argname

class ArgumentsTypeError(Exception):

	def __init__(self, argname):
		self.argname = argname



#!/usr/bin/env python
# -*- coding: utf-8  -*-

import sys
sys.path.insert(0, r"../datalet")

import os
import unittest

import datalet.storage.facade as sfd
from tests.testing import Testing

class FacadeTest(unittest.TestCase):

	def setUp(self):
		self.tmpdir = r"tests/test_data/tmp/"
		self.classname = self.__class__.__name__
		self.separator = "$"

	def tearDown(self):
		pass

	# def test_create_file(self):
	# 	for type in sfd.StorageTypes:
	# 		if type == sfd.StorageTypes.DATABASE:
	# 			continue
    #
	# 		testfile = self.classname + self.separator + self.get_func_name() + "." + type.name.lower()
	# 		testfile_fullpath = self.tmpdir + testfile
	# 		if sfd.exists_file(testfile_fullpath):
	# 			sfd.remove_file(testfile_fullpath, force = True)
    #
	# 		sfd.create_file(testfile_fullpath)
	# 		self.assertTrue(os.path.exists(testfile_fullpath))
    #
	# def test_read_file(self):
	# 	for type in sfd.StorageTypes:
	# 		if type == sfd.StorageTypes.DATABASE:
	# 			continue
    #
	# 		testfile_fullpath = r"tests/test_data/test." + type.name.lower()
	# 		dat = sfd.read_file(testfile_fullpath, sheetIndex = 0)
	# 		print(dat[:4])
	# 		self.assertTrue(len(dat) > 0)
    #
	# def test_write_file(self):
	# 	data = [["姓名", "年龄", "班级"], ["小花", 13, 3], ["小蕊", 14, 4]]
	# 	for type in sfd.StorageTypes:
	# 		if type == sfd.StorageTypes.DATABASE:
	# 			continue
	# 		testfile = self.classname + self.separator + self.get_func_name() + "." + type.name.lower()
	# 		testfile_fullpath = self.tmpdir + testfile
	# 		if not sfd.exists_file(testfile_fullpath):
	# 			sfd.create_file(testfile_fullpath, force = True)
	# 		try:
	# 			sfd.write_file(testfile_fullpath, data = data, overwrite = False, encoding = "gbk", sheetIndex = 0)
	# 		except NotImplementedError as err:
	# 			pass
	# 		finally:
	# 			pass

def suite():
	suite = unittest.TestSuite()
	suite.addTest(FacadeTest("test_create_file"))
	suite.addTest(FacadeTest("test_read_file"))
	suite.addTest(FacadeTest("test_write_file"))
	return suite

if __name__ == "__main__":
	unittest.main(defaultTest = "suite")



#!/usr/bin/env python
# -*- coding: utf-8  -*-

import unittest

class ModelMetaClass(type):
	def __new__():
		# get all columns
		attrs["getColumns"] = None







class DbTableStorage(Storage, metaclass = ModelMetaClass):

	from sqlalchemy import create_engine
	from sqlalchemy import text

	def __delete(self):
		pass

	def __delete_cascade(self):
		pass

	def __drop(self):
		pass

	def __drop_cascade(self):
		pass

	def clear(self, force = False):
		if force == False:
			self.__delete()
		else:
			self.__delete_cascade()


	def remove(self, force = False):
		if force == False:
			self.__drop()
		else:
			self.__drop_cascade()


	def __create_if_not_exist(self):
		pass

	def __create(self):
		pass


class Column(object):
	pass


class DbColumn(Column):
	pass

class BankModel(DbTableStorage):
	__schemaname__ = "testdb"
	__tablename__ = "banks"

	code = DbColumn(name = "code", nullable = False)
	name = DbColumn(name = "name", nullable = False)

class CardFactModel(DbTableStorage):
	__schemaname__ = "testdb"
	__tablename__ = "card_fact_tbl"

	date = DbColumn(name="date", nullable = True)
	src_type = DbColumn(name="type", nullable = True)
	bank_code = DbColumn(name = "bank_code")

	batch_code = DbCOlumn(name = "batch_code")



class SimpleFlowTest(unittest.TestCase):
	def setUp(self):
		pass

	def tearDown(self):
		pass

	def test_from_excel_to_postgres(self):
		# extract
		file_path = "data/test.xlsx"
		sheetIndex = 0
		df_src = excelExtractor.extract(filepath = file_path, sheetIndex = 0)

		# transform
		#card_fact =

		# load
		#db_url = ""
		#db_username = ""
		#db_userpwd = ""
		#loadRules = [
		#	Rule(column = "日期", type = Types.Date, targetField = "date", emptyHandler, unmatchHandler)
		#	, Rule(column = "类型", type = Types.Enum, targetField = "type", emptyHandler, unmatchHandler, saveRawField = "type_raw")
		#	, Rule(column = "银行", type = Types.String, targetField = "bank", emptyHandler, unmatchHandler, saveRawField = "bank_raw")
		#	, Rule(column = "卡类型", type = Types.Enum, targetField = "cardtype", emptyHandler, unmatchHandler, saveRawField = "cardtype_raw")
		#	, Rule(column = "用户数", type = Types.Integer, targetField = "usersCount", emptyHandler, unmatchHandler)
		#	, Rule(column = "卡数", type = Types.Integer, targetField = "cardsCount", emptyHandler, unmatchHandler)
		#]
		#tableLoadRule = TableLoadRule(targetSchemaName = "jd_sys", targetTableName="users", autoCreateSchema = True, appendType = AppendType.AppendAfterCleanCascade)
		#tableLoadRule.addColumnRule(ColumnRule(sourceColumn = "日期", datatype = DataTypes.Date, targetField = "date", emptyHandler = ignoreEmptyHandler, unmatchHandler = alertUnmatchHander))

		#loader = Loader(data = dat ,rules = loadRules, recreateSchema = True)
		# self.assertEqual(1, 2)


def suite():
	suite = unittest.TestSuite()
	suite.addTest(SimpleFlowTest("test_from_excel_to_postgres"))
	return suite

if __name__ == "__main__":
	unittest.main(defaultTest = "suite")



#!/usr/bin/env python
# -*- coding: utf-8  -*-

def create_pgsql_sqlalchemy_dburl(host, dbname, user, pwd, port = 5432):
	return "postgresql+psycopg2://%s:%s@%s:%d/%s" % (user, pwd, host, port, dbname)

def create_mysql_sqlalchemy_dburl(host, dbname, user, pwd, port = 3306):
	return "mysql+pymysql://%s:%s@%s:%d/%s?charset=utf8" % (user, pwd, host, port, dbname)

def create_oracle_sqlalchemy_dburl(host, dbname, user, pwd, port = 1521):
	return NotImplementedError

def create_sqlite_sqlalchemy_dburl(host, dbname, user, pwd, port = 5432):
	return NotImplementedError

def create_mssql_sqlalchemy_dburl(host, dbname, user, pwd, port = 1433):
	return NotImplementedError



#!/usr/bin/env python
# -*- coding: utf-8  -*-

import os
import os.path
from abc import ABCMeta,abstractmethod

from datalet.storage.file_storage import FileStorage
from datalet.storage.exceptions import *

class BinFileStorage(FileStorage, metaclass = ABCMeta):
	"""
	binary file
	"""

	def __init__(self, location = None):
		super().__init__(location)

	@abstractmethod
	def create(self, force = False):
		pass

	@abstractmethod
	def clear(self, force = False):
		pass

	@abstractmethod
	def write(self, data, force = True, overwrite = False, include_header = True, encoding = 'utf-8'):
		pass

	@abstractmethod
	def read(self, limit = None, encoding = 'utf-8'):
		pass



#!/usr/bin/env python
# -*- coding: utf-8  -*-

import os
import os.path
import re
from html.parser import HTMLParser

from datalet.data import *
from datalet.storage.text_file_storage import TextFileStorage
from datalet.storage.exceptions import *

class HtmlStorage(TextFileStorage):

    def __init__(self, location = None, content = None):
        super().__init__(location)
        self.content = content

    def write(self, data, overwrite = True):
        raise NotImplementedError

    def read(self, limit = -1, encoding = "utf-8"):
        htmlContent = None
        if self.content is not None:
            htmlContent = self.content
        else:
            if self.location is not None:
                 with open(self.location, "r", encoding = encoding) as file:
                    htmlContent = file.read()
            else:
                raise ArgumentsAbsenceError("The follow arguments must be specified one:  location=%s, content=%s" % \
                    (self.location, self.content))

        htmlParser = SimpleHtmlParser()
        htmlParser.feed(htmlContent)
        tablesData = htmlParser.get_tables()
        return tablesData


class RegExpHtmlParser(object):

    # regular expression
    REG_TABLE = re.compile(r'<table.*?>.*?</table>', re.M | re.I | re.S)
    REG_TR = re.compile(r'<tr.*?>.*?</tr>', re.M | re.I | re.S)
    REG_TD = re.compile(r'<td.*?>.*?</td>', re.M | re.I | re.S)
    REG_TD_VAL = re.compile(r'<td.*>(.*)</td>')
    REG_TH = re.compile(r'<th.*?>.*?</th>', re.M | re.I | re.S)
    REG_TH_VAL = re.compile(r'<th.*>(.*)</th>')
    REG_H3 = re.compile(r'<h3.*?>.*?</h3>', re.M | re.I | re.S)

    def __init__(self):
        pass

    def parse(self, htmlContent):
        tableHtmls = RegExpHtmlParser.REG_TABLE.findall(htmlContent)
        for tableHtml in tableHtmls:
            trHtmls = RegExpHtmlParser.REG_TR.findall(tableHtml)
            for trHtml in trHtmls:
                thHtmls = RegExpHtmlParser.REG_TH.findall(trHtml)
                tdHtmls = RegExpHtmlParser.REG_TD.findall(trHtml)
                for thHtml in thHtmls:
                    pass

                for tdHtml in tdHtmls:
                    pass


class SimpleHtmlParser(HTMLParser):

    def __init__(self):
        super().__init__()
        self.__dataset = DataSet()
        self.__datatable = DataTable()
        self.__datarow = DataRow()
        self.__appendable = False

    def handle_starttag(self, tag, attrs):
        if tag == "table":
            self.__datatable = DataTable()
        elif tag == "tr":
            self.__datarow = DataRow()
        elif tag == "th" or tag == "td":
            self.__appendable = True
        else:
            pass

    def handle_endtag(self, tag):
        if tag == "table":
            self.__dataset.append(self.__datatable)
        elif tag == "tr":
            self.__datatable.append(self.__datarow)
        elif tag == "th" or tag == "td":
            self.__appendable = False
        else:
            pass

    def handle_data(self, data):
        if self.__appendable == True:
            self.__datarow.append(data.strip())

    def get_dateset(self):
        return self.__dataset



#!/usr/bin/env python
# -*- coding: utf-8  -*-

from py4j.java_gateway import JavaGateway, GatewayParameters

class Client(object):

	def __init__(self, server_port = 25333):
		self.__server_port = server_port
		self.__gateway = JavaGateway(gateway_parameters=GatewayParameters(port = server_port))
		self.__entry_point = self.__gateway.entry_point

	def ping(self, msg):
		return self.__entry_point.ping(msg)

	def query_by_mdx(self, olap_driver, olap_url, olap_catalog, olap_mdx
		, db_driver, db_username, db_userpwd):
		return self.__entry_point.query_by_mdx(olap_driver, olap_url, olap_catalog, olap_mdx
		, db_driver, db_username, db_userpwd)


#!/usr/bin/env python
# -*- coding: utf-8  -*-

from enum import Enum, IntEnum, unique

@unique
class StorageTypes(IntEnum):
	CSV = 10101
	HTML = 10102
	HTM = 10103
	EML = 10104
	TSV = 10105
	XLS = 10201
	XLSX = 10202
	DATABASE = 10301


#!/usr/bin/env python
# -*- coding: utf-8  -*-

import email
import os.path

from datalet.storage.text_file_storage import TextFileStorage
from datalet.storage.html_storage import HtmlStorage

class EmlStorage(TextFileStorage):

	def __init__(self, location = None):
		super().__init__(location)

	def write(self, data, overwrite = True):
		raise NotImplementedError

	def read(self, limit = -1, encoding = "utf-8"):
		html = self.__get_html(encoding = encoding)
		hs = HtmlStorage(location = None, content = html)
		tables = hs.read(limit = limit, encoding = encoding)
		return tables

	def __get_html(self, encoding):
		html = None
		with open(self.location, encoding = encoding) as file:
			msg = email.message_from_file(file)
			for part in msg.walk():
				if not part.is_multipart():
					r = part.get_payload(decode=True)
					html = r.decode("UTF8")
			file.close()
		return html



#!/usr/bin/env python
# -*- coding: utf-8  -*-

from inspect import signature

from datalet.storage.excel_storage import ExcelStorage
from datalet.storage.csv_storage import CsvStorage
from datalet.storage.tsv_storage import TsvStorage
from datalet.storage.eml_storage import EmlStorage
from datalet.storage.html_storage import HtmlStorage
from datalet.storage.dbtable_storage import DbTableStorage
from datalet.storage.storage_types import StorageTypes
from datalet.storage.exceptions import *

__ext_2_storage_func = {StorageTypes.XLS: ExcelStorage,
				 StorageTypes.XLSX: ExcelStorage,
				 StorageTypes.CSV: CsvStorage,
				 StorageTypes.TSV: TsvStorage,
				 StorageTypes.HTML: HtmlStorage,
				 StorageTypes.HTM: HtmlStorage,
				 StorageTypes.EML: EmlStorage}


def get_storage_func(filepath = None, type = None):
	"""
	Get storage's construct function.
	"""
	if type is not None:
		if isinstance(type, StorageTypes):
			return __ext_2_storage_func[type]
		elif isinstance(type, str):
			return __ext_2_storage_func[StorageTypes[type.upper()]]
		else:
			raise TypeError("type")
	else:
		ext = filepath.split(".")[-1].upper()
		if StorageTypes[ext] in __ext_2_storage_func:
			return __ext_2_storage_func[StorageTypes[ext]]
		else:
			raise UnmatchExtensionError(filepath)


def call_func(func, args):
	"""
	Only pass the arguments whoes name is existed in function's paramsters.
	"""
	sig = signature(func)
	to_pass_args = {}
	for param in sig.parameters.values():
		if param.name in args:
			to_pass_args[param.name] = args[param.name]
	return func(**to_pass_args)



#!/usr/bin/env python
# -*- coding: utf-8  -*-

def flatten(o_dict, pre_fix = None):
	flattened = {}
	for key in o_dict.keys():
		if isinstance(o_dict[key], dict):
			p_flattened = flatten(o_dict[key], pre_fix = key)
			flattened = dict(flattened, ** p_flattened)
		else:
			new_key = key if pre_fix is None else pre_fix + "." + key
			flattened[new_key] = o_dict[key]
	return flattened


#!/usr/bin/env python
# -*- coding: utf-8  -*-

from enum import Enum, IntEnum, unique

@unique
class DataBaseTypes(IntEnum):
	POSTGRESQL = 10101
	MYSQL = 10102
	SQLITE = 10103
	MSSQLSERVER = 10104
	ORACLE = 10105


"""
execute the unittest file in project root directory.
"""
import sys
sys.path.insert(0, r"../datalet")

import os
import unittest

from datalet.storage import *
from tests.testing import Testing

class EmlStorageTest(unittest.TestCase):

	def setUp(self):
		self.tmpdir = r"tests/test_data/tmp/"
		self.classname = self.__class__.__name__
		self.separator = "$"
		self.ext = ".eml"

	def tearDown(self):
		pass

	# def test_read(self):
	# 	s = EmlStorage(r"tests/test_data/test.eml")
	# 	dat = s.read()
	# 	print(dat)
	# 	self.assertTrue(len(dat) > 0)

def suite():
	suite = unittest.TestSuite()
	#suite.addTest(EmlStorageTest("test_create_file_not_exists"))
	#suite.addTest(EmlStorageTest("test_create_file_exists"))
	#suite.addTest(EmlStorageTest("test_create_file_exists_force"))
	suite.addTest(EmlStorageTest("test_read"))
	#suite.addTest(EmlStorageTest("test_read_limit"))
	#suite.addTest(EmlStorageTest("test_write_append"))
	#suite.addTest(EmlStorageTest("test_write_overwrite"))
	#suite.addTest(EmlStorageTest("test_copy"))
	#suite.addTest(EmlStorageTest("test_copy_path"))
	return suite

if __name__ == "__main__":
	unittest.main(defaultTest = "suite")



#!/usr/bin/env python
# -*- coding: utf-8  -*-

from abc import ABCMeta,abstractmethod

class Storage(object, metaclass = ABCMeta):

	def __init__(self, location):
		"""Init an storage.

		Args:
			location:
				When the storage is a file, the location is filepath;
				When the storage is a database, the location is sqlalchemy url.
		"""
		self.location = location

	@abstractmethod
	def exists(self):
		"""Return True if the storage is existing, otherwise return False.

		Returns:
			Boolean
		"""
		pass

	@abstractmethod
	def create(self, force = True):
		"""Create a storage.

		Args:
			force:
				The storage will remove the old one and create a new one if force is True when the storage is existed otherwise raise a StorageExistedError.

		Returns:
			None

		Raises:

		"""
		pass

	@abstractmethod
	def clear(self, force = False):
		"""Clear the conent of storage.

		Args:
			force:
				The storage will be cleared cascadely if force is True when the storage has foreign relations, otherwise raise a ForeignRelationExistedError;
			 	The storage will do nothing if force is True when the storage is not existed otherwise raise a StorageNotFoundError.

		Returns:
			None
		"""
		pass

	@abstractmethod
	def remove(self, force = False):
		"""Remove the storage.

		Args:
			force:
				The storage will be removed cascadely if force is True when the storage has foreign relations, otherwise raise a ForeignRelationExistedError;
				The storage will do nothing if force is True when the storage is not existed otherwise raise a StorageNotFoundError.

		Returns:
			None
		"""
		pass

	@abstractmethod
	def write(self, data, force = True, overwrite = False, include_header = True, encoding = 'utf-8'):
		"""Write data to the storage.

		Args:
			data:
				The data to write. The data's type is DataTable(datalet.data.DataTable).
			force:
				The storage will be created if force is True when the storage is not existed otherwise raise a StorageNotFoundError.
			overwrite:
				Whether overwrite the existing data.
			include_header:
				Whether include the header if the data has header.

		Returns:
			None
		"""
		pass

	@abstractmethod
	def read(self, limit = None, encoding = 'utf-8'):
		"""Read data from the storage.

		Args:
			limit:
				Read all rows if limit is None, otherwise read the limit rows data.

		Returns:
			DataTable(datalet.data.DataTable)
		"""
		pass

	@abstractmethod
	def copy(self, copy_to_path = None):
		"""Copy the storage.

		Args:
			copy_to_path:
				The path to copy.

		Returns:
			None
		"""
		pass



#!/usr/bin/env python
# -*- coding: utf-8  -*-

import json
from datalet.data.exceptions import *

class DataSet(object):

	def __init__(self):
		self.__tables = [];

	def __getitem__(self, key):
		if isinstance(key, int) or isinstance(key, slice):
			return self.__tables[key]
		elif isinstance(key, str):
			return [table for table in self.__tables if table.name == key]
		else:
			raise ArgumentTypeError(key, type(str))

	def __setitem__(self, key, value):
		if isinstance(key, int) or isinstance(key, slice):
			self.__tables[key] = value
		elif isinstance(key, str):
			if len(self.__getitem__(key)) == 0:
				self.__tables.append(value)
			else:
				for table in self.__tables:
					if table.name == key:
						table = value
		else:
			raise ArgumentTypeError(key, type(str))

	def __delitem__(self, key):
		del self.__tables[key]

	def __len__(self):
		return len(self.__tables)

	def __iter__(self):
		return iter(self.__tables)

	def __contains__(self):
		pass

	def __del__(self):
		del self.__tables

	def __str__(self):
		return json.dumps(self)

	def append(self, table):
		return self.__tables.append(table)

	def insert(self, index, table):
		return self.__tables.insert(index, table)

	@property
	def tables(self):
		return self.__tables

	@property
	def table_names(self):
		return [table.name for table in self.__tables]



#!/usr/bin/env python
# -*- coding: utf-8  -*-

import json

from datalet.data.exceptions import *

class DataRow(object):

	def __init__(self, owner_table = None, data = []):
		"""
		data can be a list

		Args:
			owner_table:
				Parent table.
			data:
				Row's data.
		"""
		self.__owner_table = owner_table
		self.__data = list(data) if data is not None else []

	@property
	def owner_table(self):
		return self.__owner_table

	@owner_table.setter
	def owner_table(self, value):
		self.__owner_table = value

	#def __getattr__(self, key):
	#	pass

		#attr_val = None
		#print(self.__dict__.get("__owner_table"))
		#columns = self.__owner_table.columns
		#columnnames = [col.name for col in self.__owner_table.__columns]
		#if key in columnnames:
		#	idx = columnnames.index(key)
		#	attr_val = self.__data[idx]
		#else:
		#	raise AttributeNotFoundError(key)
		#return attr_val

	#def __setattr__(self, key, value):
	#	pass

	#def __delattr__(self, key):
	#	pass

	def __iter__(self):
		return iter(self.__data)

	def __getitem__(self, key):
		if isinstance(key, str):
			column_index = self.__owner_table.get_column_index(key)
			return self.__data[column_index]
		elif isinstance(key, int) or isinstance(key, slice):
			return self.__data[key]
		else:
			raise ArgumentTypeError(key, "must be str or int or slice type")


	def __setitem__(self, key, value):
		self.__data[key] = value

	def __delitem__(self, key):
		del self.__data[key]

	def __len__(self):
		return len(self.__data)

	def __str__(self):
		return "[" + ",".join(map(str, self.__data)) + "]"

	def append(self, value):
		self.__data.append(value)

	def to_dict(self, key_field, null_expr):
		if self.__owner_table is None:
			raise RequiredFieldNoneError("owner_table")
		if self.__owner_table.columns is None:
			raise RequiredFieldNoneError("owner_table.columns")

		row_dict = {}
		for i in range(0, len(self.__owner_table.columns)):
			col = self.__owner_table.columns[i]
			dat = self.__data[i]
			row_dict[col[key_field]] = null_expr if dat is None or str(dat).strip() == "" else dat

		return row_dict


	def to_list(self):
		return self.__data



#!/usr/bin/env python
# -*- coding: utf-8  -*-

def split_by_group_count(lst, group_count):
	'''按组个数，尽可能平均分配列表中的元素到各个组中。
	'''
	import math

	groups = []
	sorted_list = sorted(lst)
	list_size = len(sorted_list)
	if group_count >= list_size:
		groups.append(lst)
	else:
		group_size = int(list_size / group_count)
		for group_index in range(0, group_count):
			start_index = group_size * group_index
			end_index = group_size * (group_index + 1)
			end_index = list_size if group_index == (group_count - 1) else end_index
			group = sorted_list[start_index: end_index]
			groups.append(group)
	return groups

def split_by_group_size(lst, group_size):
	'''按组大小，将列表分成多个组。
	'''

	import math

	groups = []
	sorted_list = sorted(lst)
	list_size = len(sorted_list)
	if group_size >= list_size:
		groups.append(lst)
	else:
		group_count = math.ceil(list_size / group_size)
		for group_index in range(0, group_count):
			start_index = group_size * group_index
			end_index = group_size * (group_index + 1)
			end_index = list_size if group_index == (group_count - 1) else end_index
			group = sorted_list[start_index: end_index]
			groups.append(group)
	return groups


#!/usr/bin/env python
# -*- coding: utf-8  -*-

import re

REGEX_DIGIT = re.compile(r"\d+")

REGEX_INNER_SQUARE_BRACKETS = re.compile(r"\[(.*)\]")

REGEX_INNER_BRACES = re.compile(r"\{(.*)\}")

REGEX_INNER_PARENTHESE = re.compile(r"\((.*)\)")

REGEX_INNER_ANGLE_BRACKETS = re.compile(r"\<(.*)\>")


#!/usr/bin/env python
# -*- coding: utf-8  -*-

"""
execute the unittest file in project root directory.
"""
import sys
sys.path.insert(0, r"../datalet")

from datalet.data import *
import unittest
from tests.testing import Testing

class DataTableTesting(unittest.TestCase):

	def setUp(self):
		pass

	def tearDown(self):
		pass

	def test_init(self):
		dt1 = DataTable()

	def test_integer_index(self):
		dt = DataTable("mytable", [ DataColumn("name"), DataColumn(name = "sex"), DataColumn(name = "age")])
		dt.append(["zhangsan", "boy", 24])
		dt.append(["lisi", "girl", 30])
		dt.append(["wangwu", "boy", 33])
		self.assertTrue(dt[1]["sex"] == "girl")

	def test_iter(self):
		dt = DataTable("mytable", [ DataColumn("name"), DataColumn(name = "sex"), DataColumn(name = "age")])
		dt.append(["zhangsan", "boy", 24])
		dt.append(["lisi", "girl", 30])
		dt.append(["wangwu", "boy", 33])

		ret_column_names = []
		for col in dt.columns:
			ret_column_names.append(col.name)
		self.assertTrue(ret_column_names == ['name', 'sex', 'age'])

		ret_rows_data = []
		for row in dt:
			row_data = []
			for data in row:
				row_data.append(data)
			ret_rows_data.append(row_data)
		self.assertTrue(len(ret_rows_data) == 3)

	def test_to_dict(self):
		dt = DataTable("mytable", [ DataColumn("name"), DataColumn(name = "sex"), DataColumn(name = "age")])
		dt.append(["zhangsan", "boy", 24])
		dt.append(["lisi", "girl", 30])
		dt.append(["wangwu", "boy", 33])
		dt.append(["zhaoqi", None, None])
		self.assertTrue(isinstance(list(dt.to_dict())[0], dict))


	def test_append_row(self):
		dt = DataTable("mytable", [ DataColumn("name"), DataColumn(name = "sex"), DataColumn(name = "age")])
		dt.append(["zhangsan", "boy", 24])
		dt.append(("lisi", "girl", 30))
		dt.append({"name":"wangwu", "sex":"boy", "age":33})
		dt.append(["zhaoqi", None, None])
		self.assertTrue(isinstance(list(dt.to_dict())[0], dict))


	def test_get_data_by_column(self):
		dt = DataTable("mytable", [DataColumn("name"), DataColumn(name = "sex"), DataColumn(name = "age")])
		dt.append(["zhangsan", "boy", 24])
		dt.append(("lisi", "girl", 30))
		dt.append({"name":"wangwu", "sex":"boy", "age":33})
		dt.append(["zhaoqi", None, None])
		self.assertTrue(list(dt["name"]) == ['zhangsan', 'lisi', 'wangwu', 'zhaoqi'])

	def test_del_data_by_index(self):
		dt = DataTable("mytable", [DataColumn("name"), DataColumn(name = "sex"), DataColumn(name = "age")])
		dt.append(["zhangsan", "boy", 24])
		dt.append(("lisi", "girl", 30))
		dt.append({"name":"wangwu", "sex":"boy", "age":33})
		dt.append(["zhaoqi", None, None])
		self.assertTrue(len(dt) == 4)
		del dt[0]
		self.assertTrue(len(dt) == 3)


	def test_from_list(self):
		ls = [['name', 'age'], ['zhangsan', 29]]
		dt = DataTable.from_list(src_list = ls)
		self.assertTrue(dt[0]['name'] == 'zhangsan')


def suite():
	suite = unittest.TestSuite()
	suite.addTest(DataTableTesting("test_integer_index"))
	suite.addTest(DataTableTesting("test_iter"))
	suite.addTest(DataTableTesting("test_to_dict"))
	suite.addTest(DataTableTesting("test_append_row"))
	suite.addTest(DataTableTesting("test_get_data_by_column"))
	suite.addTest(DataTableTesting("test_del_data_by_index"))
	return suite

if __name__ == "__main__":
	unittest.main(defaultTest = "suite")



#!/usr/bin/env python
# -*- coding: utf-8  -*-

from .storage_types import StorageTypes

from .exceptions import *

from .storage import Storage
from .file_storage import FileStorage
from .text_file_storage import TextFileStorage
from .bin_file_storage import BinFileStorage
from .csv_storage import CsvStorage
from .tsv_storage import TsvStorage
from .excel_xls_storage import ExcelXlsStorage
from .excel_xlsx_storage import ExcelXlsxStorage
from .excel_storage import ExcelStorage
from .eml_storage import EmlStorage
from .html_storage import HtmlStorage



#!/usr/bin/env python
# -*- coding: utf-8  -*-

import json

from datalet.data.data_column import DataColumn
from datalet.data.data_row import DataRow
from datalet.data.exceptions import *

class DataTable(object):
	"""
	DataTable is read schema.
	It can stores the data in any format, but use the schema validating on read.

	EXAMPLE:

	Create DataTable:
		# create from list of list
		dt = DataTable(name = 'abc', columns = [], rows = [])

	Get Rows:
		# get first row
		dt[0]

		# get 1-5 rows
		dt[1: 6]

	Iterate Rows:
		for row in dt:
			...

	Get Columns:
		col_names = dt.column_names

	Get Columns Data:
		col_name_data = dt["name"]

	Get Cell Data:
		dt[row_index][col_name]
		dt[row_index].col_name
	"""

	def __init__(self, name = None, columns = [], rows = []):
		"""

		"""
		self.__name = name
		self.__rows = []
		self.__columns = []

		columns = list(columns)
		for col in columns:
			if isinstance(col, DataColumn):
				self.__columns.append(col)
			elif isinstance(col, str):
				self.__columns.append(DataColumn(col))
			else:
				raise ArgumentTypeError(columns)
		rows = list(rows)

		for row in rows:
			self.append(row)

	def __getitem__(self, key):
		"""
		if key is int or slice, get rows.
		if key is str, get column data.
		"""
		if isinstance(key, int) or isinstance(key, slice):
			return self.__rows[key]
		elif isinstance(key, str):
			return (row[key] for row in self.__rows)
		else:
			raise ArgumentTypeError(datarow, ["str", "int", "slice"])

	def __setitem__(self, key, value):
		self.__rows[key] = value

	def __delitem__(self, key):
		del self.__rows[key]

	def __iter__(self):
		return iter(self.__rows)

	def __len__(self):
		return len(self.__rows)

	def __str__(self):
		return json.dumps(self.__dict__)

	@property
	def name(self):
		return self.__name

	@name.setter
	def name(self, value):
		self.__name = value

	@property
	def columns(self):
		return self.__columns

	@columns.setter
	def columns(self, value):
		self.__columns = value

	@property
	def column_names(self):
		return [col.name for col in self.__columns]

	@property
	def shape(self):
		return (len(self.__rows), len(self.__columns))

	@property
	def rows_count(self):
		return len(self.__rows)


	def get_column_index(self, column):
		'''
		Get the index of column in the table.
		'''
		if isinstance(column, str):
			return self.column_names.index(column)
		elif isinstance(column, DataColumn):
			return self.__columns.index(column)
		else:
			raise ArgumentTypeError(datarow, ["str", "DataColumn"])


	def append(self, datarow, append_columns = False):
		'''
		Append row to table.
		'''
		if isinstance(datarow, DataRow):
			datarow.owner_table = self
			self.__rows.append(datarow)
		elif isinstance(datarow, list) or isinstance(datarow, tuple):
			self.__rows.append(DataRow(self, datarow))
		elif isinstance(datarow, dict):
			if self.__columns is None:
				raise RequiredFieldNoneError("columns")
			row_data = []
			for column in self.__columns:
				row_data.append(datarow.get(column.name, None))
			self.__rows.append(DataRow(self, row_data))
		else:
			raise ArgumentTypeError(datarow, ["list", "DataRow", "dict"])
		return self

	def extend(self, dt):
		'''
		Extend current table by appending all rows of another table.
		'''
		if isinstance(dt, DataTable):
			self.__rows.extend([row for row in dt])
		else:
			raise ArgumentTypeError(dt, "DataTable")
		return self

	def validate(self, *validators):
		pass

	def shift_rows_to_header(self, rows_count):
		if rows_count <= 0:
			raise OutOfRangeError("rows_count")

		header_rows = [self.__rows.pop(row_index) for row_index in range(0, rows_count)]
		zipped_headers = zip(*header_rows)
		column_names = ['&'.join(list(ziped_col_names)) for ziped_col_names in zipped_headers]
		self.__columns = [DataColumn(colname) for colname in column_names]

	def clear(self):
		''' clear the data in datatable, but not affect the columns.
		'''
		self.__rows = []


	def to_dict(self, key_field = "name", null_expr = None):
		'''
		return generator
		'''
		return (row.to_dict(key_field, null_expr) for row in self.__rows)


	def to_list(self):
		'''
		return generator
		'''
		return (row.to_list() for row in self.__rows)


	def to_pandas_dataframe(self, column_field = "name"):
		import pandas as pd
		column_names = [col[column_field] for col in self.__columns]
		return pd.DataFrame(data = list(self.to_list()), columns = column_names)

	def to_spark_dataframe(self):
		pass

	def to_spark_rdd(self):
		pass

	def from_pandas_dataframe(self, dataframe, null_expr = None):
		import pandas as pd
		if not isinstance(dataframe, pd.DataFrame):
			raise ArgumentTypeError(dataframe, type(pd.DataFrame))
		self.__columns = [DataColumn(column) for column in dataframe.columns]
		self.__rows = [DataRow(self, [null_expr if pd.isnull(d) else d for d in row]) \
				 for row in dataframe.values]

	@staticmethod
	def from_list(src_list, null_expr = None):
		"""
		Parse DataTable from list. The first element of the source list will be as the header columns.

		Args:
			src_list:
				The source list.

		Returns:
			DataTable
		"""

		columns = [DataColumn(column) for column in src_list[0]]
		rows = [DataRow(data = row) for row in src_list[1:]]
		return DataTable(name = None, columns = columns, rows = rows)



#!/usr/bin/env python
# -*- coding: utf-8  -*-

import datetime
from dateutil.relativedelta import *
import calendar

def get_week_date_range(date_val, start_weekday = 7):
	'''
	返回指定日期所在周的日期列表。

	@Args:
		date_val: date type or date like string.
		start_weekday: 周的起始天。默认为7。
			1表示以周一作为周的起始日，
			2表示以周二作为周的起始日，
			3表示以周三作为周的起始日，
			4表示以周四作为周的起始日，
			5表示以周五作为周的起始日，
			6表示以周六作为周的起始日，
			7表示以周日作为周的起始日。
	'''
	if (start_weekday < 1) or (start_weekday > 7):
		raise Exception('start_weekday argument value out of range.')
	if isinstance(date_val, str):
		date_val = datetime.datetime.strptime(date_val, '%Y-%m-%d').date()
	if not isinstance(date_val, datetime.date):
		raise Exception('date_val is not a date')
	week_date_range = []
	# 周一返回0
	cur_weekday = date_val.weekday()
	cur_weekday += 1
	# print('date_val cur_weekday is %d ' % cur_weekday)


	start_timedelta = (start_weekday - cur_weekday) if cur_weekday >= start_weekday else ( start_weekday - cur_weekday - 7)
	# print('start : %d' % start_timedelta)
	end_timedelta = start_timedelta + 6
	# print('end : %d' % end_timedelta)
	# print(date_val)
	# print(start_weekday)
	# print(start_timedelta)
	# print(end_timedelta)
	# print("date: %s, end_timedelta: %d" % (date_val, end_timedelta))
	for td in range(start_timedelta, end_timedelta + 1):
		week_date = date_val + datetime.timedelta(days = td)
		week_date_range.append(week_date)
	return week_date_range

def get_week_date_range_text(date_val, start_weekday = 7, date_format = "%m.%d", seprator = "-"):
	'''
	获取指定日期所在周的范围表示文本

	默认格式为 mm.dd-mm.dd
	'''
	d_range = get_week_date_range(date_val, start_weekday)
	return d_range[0].strftime(date_format) + seprator + d_range[-1].strftime(date_format)


def xdaterange(start, end):
	pass

def parse_date(str, format):
	pass

def format_date(d, format):
	pass



#!/usr/bin/env python
# -*- coding: utf-8  -*-


from gateway_lanucher import GatewayLanucher

if __name__ == "__main__":
	lanucher = GatewayLanucher()
	try:
		lanucher.start()
	except Exception as err:
		print(err)
	#finally:
	#	lanucher.stop()



#!/usr/bin/env python
# -*- coding: utf-8  -*-

"""
execute the unittest file in project root directory.
"""
import sys
sys.path.insert(0, r"../datalet")

import os
import unittest

from datalet.storage import *
import datalet.utils.inspect_utils as inspect_utils

#sys.setdefaultencoding('utf8')

class CsvStorageTest(unittest.TestCase):

	def setUp(self):
		self.tmpdir = r"tests/test_data/tmp/"
		self.classname = self.__class__.__name__
		self.separator = "$"
		self.ext = ".csv"

	def tearDown(self):
		pass

	def test_create_file_not_exists(self):
		testfile = self.classname + self.separator + inspect_utils.get_current_func_name() + self.ext
		s = CsvStorage(self.tmpdir + testfile)
		s.remove(force = True)
		s.create()
		s.remove(force = True)

	def test_create_file_exists(self):
		testfile = self.classname + self.separator + inspect_utils.get_current_func_name() + self.ext
		s = CsvStorage(self.tmpdir + testfile)
		s.remove(force = True)
		s.create()
		with self.assertRaises(StorageExistsError):
			s.create()
		s.remove(force = True)


	def test_create_file_exists_force(self):
		testfile = self.classname + self.separator + inspect_utils.get_current_func_name() + self.ext
		s = CsvStorage(self.tmpdir + testfile)
		s.remove(force = True)
		s.create()
		s.create(force = True)
		s.remove(force = True)

	def test_read(self):
		s = CsvStorage(r"tests/test_data/test.csv")
		dat = s.read(encoding = "utf-8")
		self.assertTrue(len(dat) > 0)
		print(">>> totol line count: ", len(dat))
		#for row in dat:
		#	print([cell for cell in row])


	def test_read_limit(self):
		s = CsvStorage(r"tests/test_data/test.csv")
		dat = s.read(limit = 5)
		self.assertTrue(len(dat) == 5)

	def test_write_append(self):
		testfile = self.classname + self.separator + inspect_utils.get_current_func_name() + self.ext
		s = CsvStorage(self.tmpdir + testfile)
		if not s.exists():
			s.create()
		dat = [["a", "b", "c"],[1, 2, 3]]
		s.write(data = dat)


	def test_write_overwrite(self):
		testfile = self.classname + self.separator + inspect_utils.get_current_func_name() + self.ext
		s = CsvStorage(self.tmpdir + testfile)
		s.remove(force = True)
		s.create()
		dat = [["a", "b", "c"],[1, 2, 3]]
		s.write(data = dat, overwrite = True)

	def test_copy(self):
		testfile = self.classname + self.separator + inspect_utils.get_current_func_name() + self.ext
		s = CsvStorage(self.tmpdir + testfile)
		s.remove(force = True)
		s.create()
		dat = [["a1", "b1", "c1"],[1, 2, 3]]
		s.write(data = dat, overwrite = True)
		s.copy()
		newfilename = self.classname + self.separator + inspect_utils.get_current_func_name() + FileStorage.POSTFIX + self.ext
		self.assertTrue(os.path.exists(self.tmpdir + newfilename))
		s.remove()

	def test_copy_path(self):
		testfile = self.classname + self.separator + inspect_utils.get_current_func_name() + self.ext
		s = CsvStorage(self.tmpdir + testfile)
		s.remove(force = True)
		s.create()
		dat = [["a11", "b11", "c11"],[1, 2, 3]]
		s.write(data = dat, overwrite = True)
		newpath = self.tmpdir + os.path.sep + "test_copy2.csv"
		s.copy(path = newpath)
		self.assertTrue(os.path.exists(newpath))
		s.remove()
		CsvStorage(newpath).remove()

	def test_remove(self):
		with self.assertRaises(StorageNotFoundError):
			s = CsvStorage(r"tests/test_data/test_notexists.csv")
			s.remove()
		s = CsvStorage(r"tests/test_data/test_todel.csv")
		s.create()
		s.remove()

	def test_clear(self):
		pass



#!/usr/bin/env python
# -*- coding: utf-8  -*-

from .datetime_utils import *
from .encrypt_utils import *
from .enum_utils import *
from .file_utils import *
from .list_utils import *
from .regex_utils import *
from .singleton_utils import *
from .sql_utils import *
from .str_utils import *


#!/usr/bin/env python
# -*- coding: utf-8  -*-


from datalet.storage import Storage, FileStorage, \
	TextFileStorage, BinFileStorage, \
	CsvStorage, TsvStorage, \
	ExcelStorage, ExcelXlsStorage, ExcelXlsxStorage, \
	EmlStorage, HtmlStorage



__version__ = '0.7.6'

def create_storage(location, type, **kwargs):
	if kwargs is None or len(kwargs) == 0:
		kwargs = {}



#!/usr/bin/env python
# -*- coding: utf-8  -*-

import os.path

from datalet.storage.bin_file_storage import BinFileStorage
from datalet.storage.excel_xls_storage import ExcelXlsStorage
from datalet.storage.excel_xlsx_storage import ExcelXlsxStorage
from datalet.storage.exceptions import *


class ExcelStorage(BinFileStorage):

	def __init__(self, location = None, sheet_index = None, sheet_name = None):
		super().__init__(location)

		ext = self.location.split(".")[-1].upper()
		if ext == "XLS":
			self.excel_storage = ExcelXlsStorage(location = location, sheet_index = sheet_index, sheet_name = sheet_name)
		elif ext == "XLSX":
			self.excel_storage = ExcelXlsxStorage(location = location, sheet_index = sheet_index, sheet_name = sheet_name)
		else:
			raise UnmatchExtensionError(ext)


	def create(self, force = False):
		return self.excel_storage.create(force = force)


	def clear(self, force = False):
		return self.excel_storage.clear(force = force)


	def write(self, data, force = True, overwrite = False, include_header = True, encoding = 'utf-8'):
		return self.excel_storage.write(data = data, force = force, overwrite = overwrite, include_header = include_header, encoding = encoding)


	def read(self, limit = None, encoding = 'utf-8'):
		return self.excel_storage.read(limit = limit, encoding = encoding)



#!/usr/bin/env python
# -*- coding: utf-8  -*-

import os.path
import re

from openpyxl.workbook import Workbook
from openpyxl import load_workbook

from datalet.data import *
from datalet.storage.bin_file_storage import BinFileStorage
from datalet.storage.exceptions import *


class ExcelXlsxStorage(BinFileStorage):
	"""
	Excel 2007 format Storage
	"""


	def __init__(self, location = None, sheet_index = -1, sheet_name = None):
		super().__init__(location)
		self.sheet_index = sheet_index
		self.sheet_name = sheet_name


	def __get_worksheet(self, workbook_toread):
		ws = None
		if not self.sheet_name is None:
			ws = workbook_toread.get_sheet_by_name(self.sheet_name)
		else:
			if not self.sheet_index is None:
				ws = workbook_toread.get_sheet_by_name(workbook_toread.get_sheet_names()[self.sheet_index])
			else:
				raise ArgumentsAbsenceError("The targetSheet arguments must be specified one:  sheet_index=%s, sheet_name=%s" % \
					(self.sheet_index, self.sheet_name))
		return ws


	def create(self, force = False):
		if self.exists():
			if force == False:
				raise StorageExistsError(self.location)
			else:
				self.remove(force = True)

		wb = Workbook()
		new_sheet_name = self.sheet_name if self.sheet_name is not None else ("SHEET_" + str(self.sheet_index))
		wb.create_sheet(title = new_sheet_name)
		wb.save(self.location)


	def clear(self, force = False):
		wb = load_workbook(filename = self.location)
		ws = self.__get_worksheet(wb)
		wb.remove(ws)
		new_sheet_name = self.sheet_name if self.sheet_name is not None else ("SHEET_" + str(self.sheet_index))
		wb.create_sheet(title = new_sheet_name)
		wb.save(self.location)


	def read(self, limit =  -1, data_only = True, read_only = True, encoding='utf-8'):
		dt = DataTable()
		'''
		read_only: set True when read large file.
		data_only: set True when need calculate the formatular.
		'''
		wb = load_workbook(filename = self.location, data_only = data_only, read_only = read_only)
		ws = self.__get_worksheet(wb)

		for row in ws.rows:
			dr = DataRow()
			for cell in row:
				dr.append(cell.value)
			dt.append(dr)

		# get cell data
		#for rowNum in range(1, ws.max_row + 1):
		#	if limit > -1 and rowNum > limit:
		#		break

		#	rowdata = []
		#	for colNum in range(1, ws.max_column + 1):
		#		cellval = ws.cell(row = rowNum, column= colNum).value
		#		rowdata.append(cellval)
		#	tabledata.append(rowdata)

		return dt

	def write(self, data, overwrite = False, write_only = True, write_header = True):
		'''
		use write_only mode to write large data to file.
		'''
		ILLEGAL_CHARACTERS_RE = re.compile(r'[\000-\010]|[\013-\014]|[\016-\037]')
		# if overwrite == True:
			# self.clear(force = True)
		if write_only == True:
			# data = [] if data is None else data
			# if self.exists():
				# old_data = self.read()
				# old_data.extend(data)
				# data = old_data
			#self.remove(force = True)
			wb = Workbook(write_only = True)
			ws = wb.create_sheet(title = self.sheet_name)
			if write_header == True:
				ws.append(data.column_names)
			for row in data:
				new_row = []
				for data in row:
					new_data = str(data) if data is not None else ''
					if ILLEGAL_CHARACTERS_RE.match(new_data) is not None:
						new_data = "*"
					new_row.append(new_data)
				try:
					ws.append(list(map(str, new_row)))
				except Exception as e:
					print(row)
					print(e)
					raise e
			wb.save(self.location)
		else:
			wb = load_workbook(filename = self.location)
			ws = self.__get_worksheet(wb)
			datarow_start_index = 1
			if write_header == True:
				for col_header_index in range(0, len(data.column_names)):
					cell_header_data = data.column_names[col_header_index]
					ws.cell(row = datarow_start_index, column = col_header_index + 1).value = cell_header_data
				datarow_start_index += 1
			for rowIndex in range(0, len(data)):
				rowdata = data[rowIndex]
				for colIndex in range(0, len(rowdata)):
					celldata = rowdata[colIndex]
					ws.cell(row = rowIndex + datarow_start_index, column = colIndex + 1).value = celldata
			wb.save(self.location)



#!/usr/bin/env python
# -*- coding: utf-8  -*-

"""
execute the unittest file in project root directory.
"""
import sys
sys.path.insert(0, r"../datalet")

import unittest

from datalet.storage import *
from tests.testing import Testing

class FileStorageTest(unittest.TestCase):

	def setUp(self):
		pass

	def tearDown(self):
		pass

	def test_instance(self):
		with self.assertRaises(TypeError):
			s = FileStorage() # can not instance

def suite():
	suite = unittest.TestSuite()
	suite.addTest(FileStorageTest("test_instance"))
	return suite

if __name__ == "__main__":
	unittest.main(defaultTest = "suite")



#!/usr/bin/env python
# -*- coding: utf-8  -*-

class ArgumentTypeError(Exception):

	def __init__(self, arg_value, param_type):
		self.arg_value = arg_value
		self.param_type = param_type

class ItemNotFoundError(Exception):

	def __init__(self, item):
		self.item = item

class AttributeNotFoundError(Exception):

	def __init__(self, attr_name):
		self.attr_name = attr_name


class RequiredFieldNoneError(Exception):

	def __init__(self, filed_name):
		self.field_name = field_name

class OutOfRangeError(Exception):

	def __init__(self, field_name):
		self.field_name = field_name


#!/usr/bin/env python
# -*- coding: utf-8  -*-

from abc import ABCMeta,abstractmethod

class Validator(object, metaclass = ABCMeta):
	
	@abstractmethod
	def validate(self, columns, datarow):
		pass


class DataTypeValidator(Validator):
	
	def validate(self, columns, datarow):
		pass


#!/usr/bin/env python
# -*- coding: utf-8  -*-

def detect_charset(filepath):
	'''检测文件的字符集，并提供置信度。
	'''
	import cchardet as chardet
	charset = None
	with open(filepath, "rb") as f:
		charset = chardet.detect(f.read())
	return charset

def detect_encoding(filepath):
	'''检测文件的编码。
	'''
	cset = detect_charset(filepath)
	encoding = cset["encoding"]
	confidence = cset["confidence"]
	if confidence < 0.9:
		encoding = "utf-8" if encoding == "WINDOWS-1252" else encoding
	return encoding

def try_read(filepath, raise_on_fail = True):
	'''尝试读取文件。
		首先尝试使用常用字符编码方式依次读取文件，
		如果文件没有被正确读取，使用检测的编码读取文件，
		如果文件没有被正确读取，使用codecs忽略不可用字节进行读取。
	Args:
		raise_on_fail: 经过尝试后文件没有被正常读取后抛出异常。默认为True。
	Returns:
		文件内容
	'''

	filetext = None
	read_succeed = False

	try_traces = []
	# 尝试使用常用字符编码读取文件
	common_encodings = ["utf-8", "gbk", "gb2312", "ascii", "utf_8_sig"]
	for encoding in common_encodings:
		try_read = {"try_method": "open", "encoding": encoding}
		try:
			with open(filepath, "r", encoding = encoding) as file:
				filetext = file.read()
				read_succeed = True
		except Exception as e:
			try_read["exception"] = str(e)
		finally:
			try_traces.append(try_read)
		if read_succeed == True:
			break

	# 尝试使用检测的编码读取文件
	if read_succeed == False:
		decteded_encoding = detect_encoding(filepath)
		try_read = {"try_method": "open", "encoding": decteded_encoding}
		try:
			with open(filepath, "r", encoding = decteded_encoding) as file:
				filetext = file.read()
				read_succeed = True
		except Exception as e:
			try_read["exception"] = str(e)
		finally:
			try_traces.append(try_read)

	# 尝试使用codecs忽略不可用字节进行读取
	if read_succeed == False:
		try_read = {"try_method": "codecs.open", "encoding": "utf-8"}
		import codecs
		try:
			with codecs.open(filepath, 'r', 'utf-8', 'ignore') as file:
				filetext = file.read()
				read_success = True
		except Exception as e:
			try_read["exception"] = str(e)
		finally:
			try_traces.append(try_read)

	if filetext is None and read_succeed == False and raise_on_fail == True:
		raise Exception(("try to read file failed. try traces: %s, filetext: %s") % (try_traces, filetext))

	return filetext


def convert_encoding(filepath, src_encoding = None, target_encoding = "utf-8", inplace = False):
	'''转换编码格式
	默认情况下，自动识别原始文件编码格式，将文件内容存储为UTF-8格式。

	Args:
		src_encoding: 原始文件编码格式
		target_encoding: 目标编码格式
		inplace: Default False. If set True, will replace the old file.

	Returns:
		the saved filepath.
	'''
	if src_encoding is None:
		import codecs
		src_charset = detect_charset(filepath)
		src_encoding = src_charset["encoding"]
		src_encoding = "gb2312" if src_encoding == "WINDOWS-1252" else src_encoding
	target_encoding = "utf-8" if target_encoding is None else target_encoding
	text = None
	with codecs.open(filepath, "r", encoding = src_encoding) as file:
		text = file.read()
	tosave_filepath = None
	if inplace == False:
		import os
		(dirname, filefullname) = os.path.split(filepath)
		(filename, extname) = filefullname.split('.')
		new_filename = filename + "_" + target_encoding
		new_filefullname = new_filename + "." + extname
		tosave_filepath = os.path.join(dirname, new_filefullname)
	else:
		tosave_filepath = filepath
	if text is not None:
		with codecs.open(tosave_filepath, "w", encoding = target_encoding) as file:
			file.write(text)
		return tosave_filepath
	else:
		raise Exception



#!/usr/bin/env python
# -*- coding: utf-8  -*-

"""
execute the unittest file in project root directory.
"""
import sys
sys.path.insert(0, r"../datalet")

import unittest

import pandas as pd

from datalet.data import *
from datalet.storage import *
from datalet.storage.dbtable_storage import *
import datalet.storage.facade as sfd
from sqlalchemy import create_engine

mysql_db_url = create_mysql_dburl("localhost", "abc", "abc", "123456", port = 8866)
engine = create_engine(mysql_db_url)

#sql_yueyuefa = "" \
#" select cp.id, cp.uid, cp.corp, cp.status, cp.type, cp.fund, cp.refund, cp.paidfund, cp.service, cp.paidservice, cp.penalty, cp.paidpenalty, cp.paytime, cp.expiredate, cp.period, cp.applyuid from credit_payment cp;"
# sql_yueyuefa = "" \
# " select *, (t1.not_paidfund + t1.not_paidservice + t1.not_paidpenalty) as not_sum from (" \
# " select cp.id, cp.uid, cp.corp, cp.status, cp.type, " \
# " 	cp.fund, cp.refund, cp.paidfund, cp.service, " \
# "     cp.paidservice, cp.penalty, " \
# "     cp.paidpenalty, cp.paytime, cp.expiredate, " \
# "     cp.period, cp.applyuid ," \
# "     cp.clearofftime," \
# "     (cp.fund - cp.paidfund - cp.refund) as not_paidfund," \
# "     (cp.service - cp.paidservice) as not_paidservice," \
# "     (cp.penalty - cp.paidpenalty) as not_paidpenalty," \
# "     case cp.status" \
# " 		when 1 then 0" \
# "         when 2 then 0" \
# "         when 3 then floor((UNIX_TIMESTAMP(cp.clearofftime) - UNIX_TIMESTAMP(cp.expiredate)) / (60*60*24))" \
# "         when 4 then -1" \
# "         else null" \
# " 	end as expired_days" \
# " from credit_payment cp) t1 " \
#" where t1.status = 4";

sql_yueyuefa = "" \
" select full_table.*,  " \
" 		(full_table.not_paidfund + full_table.not_paidservice + full_table.not_paidpenalty) as not_sum,  " \
" 		ai.acct_name, ai.filter_company_name, ai.trade1_name, ai.trade2_name, year(full_table.paytime) as pay_year  " \
" from  " \
 " (  " \
" 	select t1.* from " \
" 		(select cp.id, cp.uid, cp.corp, cp.status, cp.type,   " \
" 			cp.fund, cp.refund, cp.paidfund, cp.service,   " \
" 			 cp.paidservice, cp.penalty,   " \
" 			 cp.paidpenalty, cp.paytime, cp.expiredate,   " \
" 			 cp.period, cp.applyuid ,  " \
" 			 cp.clearofftime,  " \
" 			 (cp.fund - cp.paidfund - cp.refund) as not_paidfund,  " \
" 			 (cp.service - cp.paidservice) as not_paidservice,  " \
" 			 (cp.penalty - cp.paidpenalty) as not_paidpenalty,  " \
" 			 case cp.status  " \
" 				when 1 then 0  " \
" 				 when 2 then 0  " \
" 				 when 3 then floor((UNIX_TIMESTAMP(cp.clearofftime) - UNIX_TIMESTAMP(cp.expiredate)) / (60*60*24))  " \
" 				 when 4 then -1  " \
" 				 else null  " \
" 			end as expired_days  " \
" 		from yueyuefa.credit_payment cp) t1   " \
 " ) " \
" full_table  " \
" left join yueyuefa.ods_fc_acct_info ai on full_table.uid = ai.acct_id "

result = engine.execute(text(sql_yueyuefa), {})
ret_all = result.fetchall()
#first = ret_all[:5]
#print(first)

dt_zhouzhouzhuan = DataTable("table_zhouzhouzhuan",
	DataColumn(name = "id", caption = "信用加款流水号"),
	DataColumn(name = "uid", caption = "账户ID"),
	DataColumn(name = "acct_name", caption = "账户名称"),
	DataColumn(name = "filter_company_name", caption = "客户公司名称"),
	DataColumn(name = "trade1_name", caption = "一级行业"),
	DataColumn(name = "trade2_name", caption = "二级行业"),
	DataColumn(name = "corp", caption = "所属分公司"),
	DataColumn(name = "status", caption = "当前支付状态"),
	DataColumn(name = "type", caption = "信用支付属性"),
	DataColumn(name = "fund", caption = "信用加款金额"),
	DataColumn(name = "refund", caption = "信用退款金额"),
	DataColumn(name = "paidfund", caption = "已还信用款额"),
	DataColumn(name = "not_paidfund", caption = "未还信用款额"),
	DataColumn(name = "service", caption = "手续费"),
	DataColumn(name = "paidservice", caption = "已还手续费"),
	DataColumn(name = "not_paidservice", caption = "未还手续费"),
	DataColumn(name = "penalty", caption = "违约金"),
	DataColumn(name = "paidpenalty", caption = "已还违约金"),
	DataColumn(name = "not_paidpenalty", caption = "未还违约金"),
	DataColumn(name = "not_sum", caption = "欠款总额"),
	DataColumn(name = "paytime", caption = "信用加款时间"),
	DataColumn(name = "expiredate", caption = "还款截止时间"),
	DataColumn(name = "period", caption = "授信账期"),
	DataColumn(name = "expired_days", caption = "逾期天数"),
	DataColumn(name = "applyuid", caption = "申请客服"),
	DataColumn(name = "pay_year", caption = "支付年份"))

for row in ret_all:
	dt_zhouzhouzhuan.append(dict(row))

print(len(dt_zhouzhouzhuan))

sfd.write_file(r"D:\zhouzhouzhuan_acct.xlsx", data = dt_zhouzhouzhuan, sheetIndex = 0, force = True, overwrite=True)

# es_yueyuefa = sfd.create_file(r"D:\yueyuefa.xlsx", sheetIndex = 0, force = True)
# if es_yueyuefa.exists_file():
	# es_yueyuefa.remove()
# es_yueyuefa.write(data = dt_yueyuefa)



#!/usr/bin/env python
# -*- coding: utf-8  -*-

from .data_column import DataColumn
from .data_row import DataRow
from .data_table import DataTable
from .data_set import DataSet
from .exceptions import *


"""
execute the unittest file in project root directory.
"""
import sys
sys.path.insert(0, r"../datalet")

import os
import unittest
import time

from datalet.storage.temporarize import Temporarize
from tests.testing import Testing


class TemporarizeTesting(unittest.TestCase):
	def setUp(self):
		self.tmpdir = r"tests/test_data/tmp/"
		self.classname = self.__class__.__name__
		self.separator = "$"
		self.ext = ".csv"

	def tearDown(self):
		pass

	# def test_used_by_with(self):
	# 	testfile = self.classname + self.separator + self.ext
	# 	testfile_fullpath = self.tmpdir + testfile
	# 	with Temporarize(location = testfile_fullpath) as tmp:
	# 		if tmp is not None:
	# 			self.assertTrue(tmp.exists())
	# 			time.sleep(5)

	def test_used_not_by_with(self):
		pass


def suite():
	suite = unittest.TestSuite()
	suite.addTest(TemporarizeTesting("test_used_by_with"))
	suite.addTest(TemporarizeTesting("test_used_not_by_with"))
	return suite

if __name__ == "__main__":
	unittest.main(defaultTest = "suite")



#!/usr/bin/env python
# -*- coding: utf-8  -*-

import os
import sys
import subprocess
import shutil

'''
compile:
>>> javac -encoding utf-8 -Djava.ext.dirs=./lib -d ./build/classes  ./src/com/ilanever/dozer/olap/*.java

-encoding: 指定编码
-Djava.ext.dirs: 指定编译的时候调用的类
-d：要生成到的目录

package:
>>> jar cvfm ./bin/com.ilanever.dozer.olap.jar MANIFEST.MF -C ./build/classes .

"-C ./build/classes ."： 把./build/classes中的所有文件归档到jar文件中。

run:
>>> java -jar -Djava.ext.dirs=./lib ./bin/com.ilanever.dozer.olap.jar


'''

class GatewayLanucher(object):
	
	def __init__(self, port = 25333, force = True):
		self.__cur_dir = os.path.split(os.path.realpath(__file__))[0]
		self.__java_root_dir = os.path.join(self.__cur_dir, "java")
		self.__java_src_dir = os.path.join(self.__java_root_dir, "src")
		self.__java_lib_dir = os.path.join(self.__java_root_dir, "lib")
		self.__build_root_dir = os.path.join(self.__java_root_dir, "build")
		if not os.path.exists(self.__build_root_dir):
			os.makedirs(self.__build_root_dir)
		self.__build_classes_dir = os.path.join(self.__build_root_dir, "classes")
		if not os.path.exists(self.__build_classes_dir):
			os.makedirs(self.__build_classes_dir)
		self.__java_bin_dir = os.path.join(self.__java_root_dir, "bin")
		if not os.path.exists(self.__java_bin_dir):
			os.makedirs(self.__java_bin_dir)
		self.__target_jar = os.path.join(self.__java_bin_dir, "com.ilanever.dozer.olap.jar")
		self.__manifest_path = os.path.join(self.__java_root_dir, "MANIFEST.MF")
		#self.__run_process = None
		self.__port = port
		self.__force = force

	def __check_env(self):
		not_found = []
		try:
			subprocess.call(["javac"], stdout = subprocess.PIPE, stderr = subprocess.PIPE)
		except FileNotFoundError as err:
			not_found.append("javac")

		try:
			subprocess.call(["jar"], stdout = subprocess.PIPE, stderr = subprocess.PIPE)
		except FileNotFoundError as err:
			not_found.append("jar")

		try:
			subprocess.call(["java"], stdout = subprocess.PIPE, stderr = subprocess.PIPE)
		except FileNotFoundError as err:
			not_found.append("java")

		return not_found

	def __remove_files(self):
		if os.path.exists(self.__target_jar):
			os.remove(self.__target_jar)
		if os.path.exists(self.__build_classes_dir):
			shutil.rmtree(self.__build_classes_dir)
			os.mkdir(self.__build_classes_dir)

	def __compile(self):
		print("start to compile...")
		try:
			for root, dirs, files in os.walk(self.__java_src_dir): 
				for file in files:
					cmd = "javac -encoding utf-8 -Djava.ext.dirs=%s -d %s -sourcepath %s -classpath %s %s" % \
						(self.__java_lib_dir, self.__build_classes_dir, self.__java_src_dir, \
						self.__build_classes_dir ,os.path.join(root, file))
					print(cmd)
					# or use subprocess.call()
					p = subprocess.Popen(cmd)
					p.wait()

		except Exception as err:
			raise err

	def __package(self):
		print("start to package...")
		try:
			cmd = "jar cvfm %s %s -C %s ." % (self.__target_jar, self.__manifest_path, self.__build_classes_dir)
			print(cmd)
			return subprocess.Popen(cmd)
		except Exception as err:
			raise err

	def __run(self):
		print("start to run...")
		try:
			cmd = "java -jar -Djava.ext.dirs=%s %s" % (self.__java_lib_dir, self.__target_jar)
			print(cmd)
			return subprocess.Popen(cmd)
		except Exception as err:
			raise err

	def start(self):
		not_founds = self.__check_env()
		if len(not_founds) > 0:
			raise EnvironmentError("not found in PATH: ", ", ".join(not_founds))

		if self.__force == True:
			self.__remove_files()

		self.__compile()
		pk_proc = self.__package()
		pk_proc.wait()
		run_proc = self.__run()
		run_proc.wait()

	#def stop(self):
	#	self.__run_process.kill()


#if __name__ == "__main__":
#	lanucher = GatewayLanucher()
#	lanucher.start()


#!/usr/bin/env python
# -*- coding: utf-8  -*-

from sqlalchemy import create_engine

import datalet.utils.sql_utils

sql = None
mysql_db_url = sql_utils.create_mysql_sqlalchemy_dburl(
	"10.52.27.40",
	"yueyuefa",
	"yueyuefa",
	"123456",
	port = 8866)

engine = create_engine(mysql_db_url)
resultProxy = engine.execute(sql)
for key in resultProxy.keys():
	print(key)



#!/usr/bin/env python
# -*- coding: utf-8  -*-

def singleton(cls, *args, **kw):
	instances = {}
	def __singleton():
		if cls not in instances:
			instances[cls] = cls(*args, **kw)
		return instances[cls]
	return __singleton


#!/usr/bin/env python
# -*- coding: utf-8  -*-

from datalet.storage.storage import Storage
from datalet.storage.exceptions import *
from datalet.storage._utils import *


class Temporarize(object):
	"""
	usage:
		with Temporarize("location") as temp:
			....
	"""


	def __init__(self, storage = None, location = None, **kwargs):
		self.storage = None
		if storage is not None :
			if isinstance(storage, Storage):
				self.storage = storage
			else:
				raise TypeError(storage)
		else:
			if location is not None:
				if kwargs is None or len(kwargs) == 0:
					kwargs = {}
				kwargs["location"] = location
				kwargs["force"] = True
				if isinstance(location, str):
					construct_func = get_storage_func(location)
					self.storage = call_func(construct_func, kwargs)
					call_func(self.storage.create, kwargs)
				elif isinstance(location, dict):
					#self.storage = sfd.create_db(
					raise NotImplementedError()
			else:
				raise ArgumentsAbsenceError("storage and location must be specified one.")

	def __enter__(self):
		return self.storage

	def __exit__(self, e_t, e_v, t_b):
		if self.storage is not None:
			self.storage.remove(force = True)



#!/usr/bin/env python
# -*- coding: utf-8  -*-

"""
execute the unittest file in project root directory.
"""
import sys
sys.path.insert(0, r"../datalet")
import unittest

from datalet.data import *


class DataColumnTesting(unittest.TestCase):

	def setUp(self):
		pass

	def tearDown(self):
		pass

	def test_getattr(self):
		col_name = DataColumn(name = "col_name", caption = "STUDENTNAME", data_type = type(str), nullable = True, default_value = 0)
		self.assertTrue(col_name.name == "col_name")
		self.assertTrue(col_name.caption == "STUDENTNAME")

	def test_setattr(self):
		col_name = DataColumn(name = "col_name", caption = "STUDENTNAME", data_type = type(str), nullable = True, default_value = 0)
		col_name.caption = "NAMEBIGGER"
		self.assertTrue(col_name.caption == "NAMEBIGGER")

	def test_delattr(self):
		col_name = DataColumn(name = "col_name", caption = "STUDENTNAME", data_type = type(str), nullable = True, default_value = 0)
		del col_name.caption
		self.assertTrue(col_name.caption == None)


def suite():
	suite = unittest.TestSuite()
	suite.addTest(DataColumnTesting("test_getattr"))
	suite.addTest(DataColumnTesting("test_setattr"))
	suite.addTest(DataColumnTesting("test_delattr"))
	return suite

if __name__ == "__main__":
	unittest.main(defaultTest = "suite")



#!/usr/bin/env python
# -*- coding: utf-8  -*-

import os
import os.path

from datalet.data import *
from datalet.storage.text_file_storage import TextFileStorage
from datalet.storage.exceptions import *

class TsvStorage(TextFileStorage):

	def __init__(self, location):
		super().__init__(location)


	def write(self, data, force = True, overwrite = False, include_header = True, encoding = 'utf-8'):
		"""
		See Storage's notes.
		"""
		if not self.exists():
			if force == True:
				self.create(force = True)
			else:
				raise StorageNotFoundError(self.location)

		openmode = "a" if overwrite == False else "w"
		with open(self.location, openmode, encoding = encoding) as file:
			if write_header == True:
				file.write('\t'.join(data.column_names))
				file.write('\n')
			for row in data:
				file.write('\t'.join(list(map(str, row))))
				file.write('\n')


	def read(self, limit = None, encoding = "utf-8"):
		"""
		See Storage's notes.
		"""
		if not self.exists():
			raise StorageNotFoundError(self.location)

		dt = DataTable()
		with open(self.location, "r", encoding = encoding) as file:
			line_index = 0
			for line in file.readlines():
				if limit is not None and line_index >= limit:
					break
				dt.append(line.rstrip('\n').split('\t'))
				line_index += 1
		return dt



#!/usr/bin/env python
# -*- coding: utf-8  -*-

import os
import os.path
from abc import ABCMeta,abstractmethod

from datalet.storage.storage import Storage
from datalet.exceptions import *

class FileStorage(Storage, metaclass = ABCMeta):

	def __init__(self, location = None):
		"""Init FileStorage

		Args:
			location: the location is location.
		"""
		super().__init__(location)


	def exists(self):
		"""
		See Storage's notes.
		"""
		return os.path.exists(self.location)


	def remove(self, force = False):
		"""
		See Storage's notes.
		"""
		if not self.exists():
			if force == False:
				raise StorageNotFoundError(self.location)
			else:
				# do nothing.
				pass
		else:
			os.remove(self.location)


	def copy(self, copy_to_path = None):
		"""
		See Storage's notes.
		"""
		if not self.exists():
			raise StorageNotFoundError(self.location)
		if copy_to_path is None:
			raise ArgumentsAbsenceError('copy_to_path')

		import shutil
		shutil.copyfile(self.location, copy_to_path)


	@abstractmethod
	def create(self, force = False):
		pass

	@abstractmethod
	def clear(self, force = False):
		pass

	@abstractmethod
	def write(self, data, force = True, overwrite = False, include_header = True, encoding = 'utf-8'):
		pass

	@abstractmethod
	def read(self, limit = None, encoding = 'utf-8'):
		pass



#!/usr/bin/env python
# -*- coding: utf-8  -*-

from setuptools import setup, find_packages

from datalet import __version__

setup(
	name = 'datalet',
	version = __version__,
	packages = find_packages(),
	keywords = ['datalet'],
	author = 'li pei',
	author_email = 'pei@ilanever.com',
	description = 'datalet is a toolkit for data analysis.',
	long_description = open('README.rst').read(),
	include_package_data = True,
	url = 'https://github.com/lipei86/datalet',
	install_requires=[
		'openpyxl>=2.5.0a3',
		'xlrd>=1.1.0',
		'xlwt>=1.3.0',
		'python-dateutil>=2.6.1',
		'cchardet>=2.1.1'
	]
	)



#!/usr/bin/env python
# -*- coding: utf-8  -*-

import os.path

import xlrd
import xlwt

from datalet.data import *
from datalet.storage.bin_file_storage import BinFileStorage
from datalet.storage.exceptions import *


class ExcelXlsStorage(BinFileStorage):
	"""
	Excel 2003 format Storage
	"""


	def __init__(self, location = None, sheet_index = -1, sheet_name = None):
		super().__init__(location)
		self.sheet_index = sheet_index
		self.sheet_name = sheet_name


	def create(self, force = False):
		if self.exists():
			if force == False:
				raise StorageExistedError(self.location)
			else:
				self.remove(force = True)

		wb_w = xlwt.Workbook(encoding='utf-8', style_compression=0)
		new_sheet_name = self.sheet_name if self.sheet_name is not None else ("SHEET_" + str(self.sheet_index))
		sheet = wb_w.add_sheet(new_sheet_name, cell_overwrite_ok = True)
		wb_w.save(self.location)


	def clear(self, force = False):
		if not self.exists():
			if force == False:
				raise StorageNotFoundError(self.location)

		raise NotImplementedError()


	def read(self, limit = None, encoding='utf-8'):
		dt = DataTable()
		wb = xlrd.open_workbook(self.location)
		ws = self.__get_worksheet(wb)

		# get cell data
		for rowIndex in range(0, ws.nrows):
			if limit > -1 and (rowIndex + 1) > limit:
				break

			row = ws.row(rowIndex)
			dr = DataRow()
			for colIndex in range(0, ws.ncols):
				cell = row[colIndex]
				cellval = cell.value
				dr.append(cellval)
			dt.append(dr)

		return dt


	def write(self, data, overwrite = False):
		raise NotImplementedError()

	def __get_worksheet(self, workbook_toread):
		ws = None
		if self.sheet_name is not None:
			ws = workbook_toread.sheet_by_name(self.sheet_name)
		else:
			if self.sheet_index is not -1:
				ws = workbook_toread.sheet_by_index(self.sheet_index)
			else:
				raise ArgumentsAbsenceError("The targetSheet arguments must be specified one:  sheetIndex=%s, sheetName=%s" % \
					(self.sheet_index, self.sheet_name))
		return ws



#!/usr/bin/env python
# -*- coding: utf-8  -*-

import os.path
import math

from sqlalchemy import create_engine, text, select, exists
from sqlalchemy.schema import CreateSchema

from datalet.storage.storage import Storage
from datalet.storage.db_types import DataBaseTypes
from datalet.storage.exceptions import *

class DbTableStorage(Storage):

	def __init__(self, dburl, table, engine_echo = False):
		location = {"dburl": dburl, "table": table}
		super().__init__(location)
		self.dburl = dburl
		self.table = table
		self.engine = create_engine(self.dburl, echo = engine_echo)
		self.table.metadata.bind = self.engine

	def get_engine(self):
		return self.engine

	def get_table(self):
		return self.table

	def __exists_schema(self, schema_name):
		txt_exists_schema = text("SELECT schema_name FROM information_schema.schemata WHERE schema_name = :schema_name")
		schemanames = self.engine.execute(txt_exists_schema, schema_name = schema_name).fetchall()
		return len(schemanames) > 0


	def create(self, force = False, partition_by = None):
		if self.exists():
			if force == True:
				self.remove(force = True)
			else:
				raise StorageExistedError(self.location)
		# create
		if not self.__exists_schema(self.table.schema):
			self.engine.execute(CreateSchema(self.table.schema))
		return self.table.create()


	def exists(self):
		return self.table.exists()


	def clear(self, force = False):
		if not self.exists():
			if force == False:
				raise StorageNotFoundError(self.location)
		return self.table.delete()


	def remove(self, force = False):
		if not self.exists():
			if force == False:
				raise StorageNotFoundError(self.location)
		else:
			self.table.drop()


	def copy(self, path = None):
		pass


	def read(self, limit =  -1):
		return self.engine.execute(self.table.select())


	def write(self, data, overwrite = False, batch_rows_count = -1):
		"""
		DataFrame.to_dict(orient = "records")
		"""
		if not self.exists():
			raise StorageNotFoundError(self.location)

		if not isinstance(data, list):
			data = list(data)

		# TODO: make it to support generator.
		if overwrite == True:
			self.clear(force = True)

		if batch_rows_count <= 0:
			return self.engine.execute(self.table.insert(), data)
		else:
			rows_count = len(data)
			batch_count = 1 if batch_rows_count >= rows_count else math.ceil(rows_count / batch_rows_count)
			# start transaction ??
			with self.engine.begin() as conn:
				trans = conn.begin()
				try:
					for batch_index in range(0, batch_count):
						start_index = batch_index * batch_rows_count
						end_index = rows_count if batch_index == (batch_count - 1) else start_index + batch_rows_count
						print("proc>>> batch index: %d, start index: %d, end index: %d" % (batch_index, start_index, end_index))

						conn.execute(self.table.insert(), data[start_index: end_index])
					trans.commit()
				except:
					trans.rollback()
					raise
			return (rows_count, batch_count)



#!/usr/bin/env python
# -*- coding: utf-8  -*-

import os
import os.path
from abc import ABCMeta,abstractmethod

from datalet.storage.file_storage import FileStorage
from datalet.storage.exceptions import *

class TextFileStorage(FileStorage, metaclass = ABCMeta):
	"""
	text file
	"""

	def __init__(self, location = None):
		super().__init__(location)


	def create(self, force = False):
		"""
		See Storage's notes.
		"""
		if self.exists():
			if force == False:
				raise StorageExistedError(self.location)
			else:
				self.remove(force = True)

		(head, tail) = os.path.split(self.location)
		if head != "" and (not os.path.exists(head)):
			os.makedirs(head)
		with open(self.location, "w", encoding="utf-8") as file:
			pass


	def clear(self, force = False):
		"""
		See Storage's notes.
		"""
		if not self.exists():
			if force == False:
				raise StorageNotFoundError(self.location)
		with open(self.location, "w", encoding="utf-8") as file:
			file.truncate()


	@abstractmethod
	def write(self, data, overwrite = False):
		pass

	@abstractmethod
	def read(self, limit = -1, encoding = "utf-8"):
		pass



#!/usr/bin/env python
# -*- coding: utf-8  -*-

"""
execute the unittest file in project root directory.
"""
import sys
sys.path.insert(0, r"../datalet")

import os
import unittest

from datalet.storage import *
from tests.testing import Testing



class ExcelStorageTest(unittest.TestCase):

	def setUp(self):
		self.tmpdir = r"tests/test_data/tmp/"
		self.classname = self.__class__.__name__
		self.separator = "$"
		self.ext = ".xlsx"

	def tearDown(self):
		pass

	# def test_create_file_not_exists(self):
	# 	testfile = self.classname + self.separator + self.get_func_name() + self.ext
	# 	s = ExcelStorage(self.tmpdir + testfile, sheetIndex = 0)
	# 	s.remove(force = True)
	# 	s.create()
	# 	s.remove(force = True)
    #
	# def test_create_file_exists(self):
	# 	testfile = self.classname + self.separator + self.get_func_name() + self.ext
	# 	s = ExcelStorage(self.tmpdir + testfile, sheetIndex = 0)
	# 	s.remove(force = True)
	# 	s.create()
	# 	with self.assertRaises(StorageExistsError):
	# 		s.create()
	# 	s.remove(force = True)
    #
    #
	# def test_create_file_exists_force(self):
	# 	testfile = self.classname + self.separator + self.get_func_name() + self.ext
	# 	s = ExcelStorage(self.tmpdir + testfile, sheetIndex = 0)
	# 	s.remove(force = True)
	# 	s.create()
	# 	s.create(force = True)
	# 	s.remove(force = True)
    #
	# def test_read(self):
	# 	s = ExcelStorage(r"tests/test_data/test.xlsx", sheetIndex = 0)
	# 	dat = s.read()
	# 	self.assertTrue(len(dat) > 0)
    #
	# def test_read_limit(self):
	# 	s = ExcelStorage(r"tests/test_data/test.xlsx", sheetIndex = 0)
	# 	dat = s.read(limit = 5)
	# 	self.assertTrue(len(dat) == 5)
    #
	# def test_write_append(self):
	# 	testfile = self.classname + self.separator + self.get_func_name() + self.ext
	# 	s = ExcelStorage(self.tmpdir + testfile, sheetIndex = 0)
	# 	if not s.exists():
	# 		s.create()
	# 	dat = [["a", "b", "c"],[1, 2, 3]]
	# 	s.write(data = dat)
    #
    #
	# def test_write_overwrite(self):
	# 	testfile = self.classname + self.separator + self.get_func_name() + self.ext
	# 	s = ExcelStorage(self.tmpdir + testfile, sheetIndex = 0)
	# 	s.remove(force = True)
	# 	s.create()
	# 	dat = [["a", "b", "c"],[1, 2, 3]]
	# 	s.write(data = dat, overwrite = True)
    #
	# def test_copy(self):
	# 	testfile = self.classname + self.separator + self.get_func_name() + self.ext
	# 	s = ExcelStorage(self.tmpdir + testfile, sheetIndex = 0)
	# 	s.remove(force = True)
	# 	s.create()
	# 	dat = [["a1", "b1", "c1"],[1, 2, 3]]
	# 	s.write(data = dat, overwrite = True)
	# 	s.copy()
	# 	newfilename = self.classname + self.separator + self.get_func_name() + FileStorage.POSTFIX + self.ext
	# 	self.assertTrue(os.path.exists(self.tmpdir + newfilename))
	# 	s.remove()
    #
	# def test_copy_path(self):
	# 	testfile = self.classname + self.separator + self.get_func_name() + self.ext
	# 	s = ExcelStorage(self.tmpdir + testfile, sheetIndex = 0)
	# 	s.remove(force = True)
	# 	s.create()
	# 	dat = [["a11", "b11", "c11"],[1, 2, 3]]
	# 	s.write(data = dat, overwrite = True)
	# 	newpath = self.tmpdir + os.path.sep + "test_copy2.xlsx"
	# 	s.copy(path = newpath)
	# 	self.assertTrue(os.path.exists(newpath))
	# 	s.remove()
	# 	ExcelStorage(newpath).remove()
    #
	# def test_remove(self):
	# 	with self.assertRaises(StorageNotFoundError):
	# 		s = ExcelStorage(r"tests/test_data/test_notexists.xlsx", sheetIndex = 0)
	# 		s.remove()
	# 	s = ExcelStorage(r"tests/test_data/test_todel.xlsx", sheetIndex = 0)
	# 	s.create()
	# 	s.remove()
    #
	# def test_clear(self):
	# 	pass


def suite():
	suite = unittest.TestSuite()
	suite.addTest(ExcelStorageTest("test_create_file_not_exists"))
	suite.addTest(ExcelStorageTest("test_create_file_exists"))
	suite.addTest(ExcelStorageTest("test_create_file_exists_force"))
	suite.addTest(ExcelStorageTest("test_read"))
	suite.addTest(ExcelStorageTest("test_read_limit"))
	suite.addTest(ExcelStorageTest("test_write_append"))
	suite.addTest(ExcelStorageTest("test_write_overwrite"))
	suite.addTest(ExcelStorageTest("test_copy"))
	suite.addTest(ExcelStorageTest("test_copy_path"))
	return suite

if __name__ == "__main__":
	unittest.main(defaultTest = "suite")



#!/usr/bin/env python
# -*- coding: utf-8  -*-

"""
execute the unittest file in project root directory.
"""
import sys
sys.path.insert(0, r"../datalet")

from datalet.data import *
import unittest

class DataRowTesting(unittest.TestCase):
	def setUp(self):
		pass

	def tearDown(self):
		pass

	def test_init(self):
		ls = ['zhangsan', 20, 'abc@abc.com']
		dr = DataRow(None, ls)
		self.assertTrue(dr[0] == 'zhangsan')

	def test_get_data_by_colname(self):
		dt = DataTable(name = None, columns = ['name', 'age', 'email'])
		ls = ['zhangsan', 20, 'abc@abc.com']
		dr = DataRow(dt, ls)
		self.assertTrue(dr['name'] == 'zhangsan')



#!/usr/bin/env python
# -*- coding: utf-8  -*-

import json

class DataColumn(object):
	
	def __init__(self, name, caption = None, data_type = type(str), nullable = True, default_value = None):
		self.name = name
		self.caption = caption
		self.data_type = data_type
		self.nullable = nullable
		self.default_value = default_value
		
	def __getattr__(self, attr_name):
		return self.__dict__.get(attr_name)

	def __setattr__(self, attr_name, attr_val):
		self.__dict__[attr_name] = attr_val

	def __delattr__(self, attr_name):
		del self.__dict__[attr_name]

	def __getitem__(self, key):
		return self.__getattr__(key)

	def __setitem__(self, key, value):
		self.__setattr__(key, value)

	def __delitem__(self, key):
		self.__delattr__(key)

	def __str__(self):
		return json.dumps(self)
